/**
 * Provides the defaults of sizes, sounds, and images for a game.
 * Also controls how keyboard input is mapped to an action in the game.
 * By subclassing this class, you could create a new "skin" for the game.
 * 
 * Do NOT modify this file.
 * 
 * @author Keith Vertanen
 *
 */
public class Config
{
	/**
	 * Translates a letter hit by the player into the action that that
	 * key causes to happen in the game. Currently this is just the
	 * cardinal direction that the hero is attempting to move in.
	 * If the character isn't a direction, returns INVALID.
	 * 
	 * @param ch character indicating direction of movement
	 * @return action to be performed
	 */
	public Action mapKeyToAction(char ch)
    {
        switch (ch)
        {
            case 'w':  return Action.NORTH;
            case 's':  return Action.SOUTH;
            case 'a':  return Action.WEST;
            case 'd':  return Action.EAST;
            default :  return Action.INVALID;                
        }
    }
    
	/**
	 * Maps a specified item type to the image that is used to display it
	 * 
	 * @param item the item to be mapped to an image
	 * @return the file name of the image
	 */
	public String mapItemToImage(Item item)
    {
        switch (item)
        {
            case KEY: return "key.png";
            case GEM: return "gem.png";
            default:  return "";
        }
    }
    
	/**
	 * Maps a specified item type to the sound file played when it is collected
	 * 
	 * @param item the item to be mapped to a sound
	 * @return the file name of the sound
	 */
	public String mapItemToCollectSound(Item item)
    {
        switch (item)
        {
            case KEY: return "key.wav";
            case GEM: return "gem.wav";
            default:  return "";
        }
    }
    
	/**
	 * Maps a specified item type to the sound file played when an item is used
	 * 
	 * @param item the item to be mapped to a sound
	 * @return the file name of the sound
	 */
	public String mapItemToUseSound(Item item)
    {
        switch (item)
        {
            case KEY: return "open.wav";
            default:  return "";
        }
    }

	/**
	 * Returns the cell size in pixels
	 * 
	 * @return the constant 32 -- the size of the cell in pixels
	 */
	public int cellPixels()
    {
        return 32;
    }
    
	/**
	 * Returns filename of the sound played when walking
	 * 
	 * @return the file name for the walk sound
	 */
	public String walkSound()
    {
        return "walk.wav";
    }
    
	/**
	 * Returns filename of the sound played when hero hits a wall
	 * 
	 * @return the file name of the hit-the-wall sound
	 */
	public String hitWallSound()
    {
        return "wall.wav";
    }
        
	/**
	 * Returns filename of the default image used for the hero
	 * 
	 * @return default file name for hero image
	 */
	public String heroImage()
    {
        return "girl.png";
    }

	/**
	 * Returns filename of the wall image
	 * 
	 * @return default file name for wall image
	 */
	public String wallImage()
    {
        return "wall.png";
    }
        
	/**
	 * Returns filename of the passageway image
	 * 
	 * @return the default file name for the passage image
	 */
	public String passageImage()
    {
        return "passage.png";
    }

	/**
	 * Returns filename of an open door image
	 * 
	 * @return the default file name for the open door image
	 */
	public String openDoorImage()
    {
        return "door_open.png";
    }

	/**
	 * Returns filename of a closed door image
	 * 
	 * @return the default file name for a closed door image
	 */
	public String closedDoorImage()
    {
        return "door_closed.png";
    }
    
}
